@@ -11,6 +11,7 @@ from dataclasses import asdict
 import click
 from click.exceptions import MissingParameter, UsageError
 from pykdebugparser.pykdebugparser import PyKdebugParser
+from pymobiledevice3.services.device_arbitration import DtDeviceArbitration
 from termcolor import colored
 
 import pymobiledevice3
@@ -533,7 +534,7 @@ def dvt_oslog(lockdown: LockdownClient, color, pid):
                 message_pid = message.process
                 # without message_type maybe signpost have event_type
                 message_type = message.message_type if hasattr(message, 'message_type') else message.event_type \
-                    if hasattr(message, 'event_type') else "unknown"
+                    if hasattr(message, 'event_type') else 'unknown'
                 sender_image_path = message.sender_image_path
                 image_name = os.path.basename(sender_image_path)
                 subsystem = message.subsystem
@@ -542,11 +543,10 @@ def dvt_oslog(lockdown: LockdownClient, color, pid):
                 if pid is not None and message_pid != pid:
                     continue
 
-                try:
-                    formatted_message = decode_message_format(message.message).decode() \
-                        if message.message else message.name
-                except Exception:
-                    print('error decoding')
+                if message.message:
+                    formatted_message = decode_message_format(message.message)
+                else:
+                    formatted_message = message.name
 
                 if color:
                     message_pid = colored(str(message_pid), 'magenta')
@@ -555,7 +555,8 @@ def dvt_oslog(lockdown: LockdownClient, color, pid):
                     image_name = colored(image_name, 'yellow')
                     message_type = colored(message_type, 'cyan')
 
-                print(f'[{subsystem}][{category}][{message_pid}][{image_name}] <{message_type}>: {formatted_message}')
+                print(f'[{subsystem}][{category}][{message_pid}][{image_name}] '
+                      f'<{message_type}>: {formatted_message}')
 
 
 @dvt.command('energy', cls=Command)
@@ -794,3 +795,33 @@ def debugserver_start_server(lockdown: LockdownClient, local_port):
     """
     attr = lockdown.get_service_connection_attributes('com.apple.debugserver.DVTSecureSocketProxy')
     TcpForwarder(lockdown, local_port, attr['Port'], attr.get('EnableServiceSSL', False)).start()
+
+
+@developer.group('arbitration')
+def arbitration():
+    """ arbitration options. """
+    pass
+
+
+@arbitration.command('version', cls=Command)
+@click.option('--color/--no-color', default=True)
+def version(lockdown: LockdownClient, color):
+    """ get arbitration version """
+    with DtDeviceArbitration(lockdown) as device_arbitration:
+        print_json(device_arbitration.version, colored=color)
+
+
+@arbitration.command('check-in', cls=Command)
+@click.argument('hostname')
+@click.option('-f', '--force', default=False, is_flag=True)
+def check_in(lockdown: LockdownClient, hostname, force):
+    """ owner check-in """
+    with DtDeviceArbitration(lockdown) as device_arbitration:
+        device_arbitration.check_in(hostname, force=force)
+
+
+@arbitration.command('check-out', cls=Command)
+def check_out(lockdown: LockdownClient):
+    """ owner check-out """
+    with DtDeviceArbitration(lockdown) as device_arbitration:
+        device_arbitration.check_out()
