@@ -43,9 +43,9 @@ IRECV_DEVICES = (
     IRecvDevice(product_type='iPhone8,2', hardware_model='n66map', board_id=0x06, chip_id=0x8003,
                 display_name='iPhone 6s Plus'),
     IRecvDevice(product_type='iPhone8,4', hardware_model='n69ap', board_id=0x02, chip_id=0x8003,
-                display_name='iPhone SE'),
+                display_name='iPhone SE (1st gen)'),
     IRecvDevice(product_type='iPhone8,4', hardware_model='n69uap', board_id=0x02, chip_id=0x8000,
-                display_name='iPhone SE'),
+                display_name='iPhone SE (1st gen)'),
     IRecvDevice(product_type='iPhone9,1', hardware_model='d10ap', board_id=0x08, chip_id=0x8010,
                 display_name='iPhone 7 (Global)'),
     IRecvDevice(product_type='iPhone9,2', hardware_model='d11ap', board_id=0x0a, chip_id=0x8010,
@@ -81,7 +81,7 @@ IRECV_DEVICES = (
     IRecvDevice(product_type='iPhone12,5', hardware_model='d431ap', board_id=0x02, chip_id=0x8030,
                 display_name='iPhone 11 Pro Max'),
     IRecvDevice(product_type='iPhone12,8', hardware_model='d79ap', board_id=0x10, chip_id=0x8030,
-                display_name='iPhone SE (2020)'),
+                display_name='iPhone SE (2nd gen)'),
     IRecvDevice(product_type='iPhone13,1', hardware_model='d52gap', board_id=0x0A, chip_id=0x8101,
                 display_name='iPhone 12 mini'),
     IRecvDevice(product_type='iPhone13,2', hardware_model='d53gap', board_id=0x0C, chip_id=0x8101,
@@ -98,6 +98,8 @@ IRECV_DEVICES = (
                 display_name='iPhone 13 mini'),
     IRecvDevice(product_type='iPhone14,5', hardware_model='d17ap', board_id=0x0A, chip_id=0x8110,
                 display_name='iPhone 13'),
+    IRecvDevice(product_type='iPhone14,6', hardware_model='d49ap', board_id=0x10, chip_id=0x8110,
+                display_name='iPhone SE (3rd gen)'),
     # iPod
     IRecvDevice(product_type='iPod1,1', hardware_model='n45ap', board_id=0x02, chip_id=0x8900,
                 display_name='iPod Touch (1st gen)'),
@@ -259,6 +261,10 @@ IRECV_DEVICES = (
                 display_name='iPad Pro 12.9-inch (5th gen, Cellular)'),
     IRecvDevice(product_type='iPad13,11', hardware_model='j523xap', board_id=0x1E, chip_id=0x8103,
                 display_name='iPad Pro 12.9-inch (5th gen, Cellular, 2TB)'),
+    IRecvDevice(product_type='iPad13,16', hardware_model='j407ap', board_id=0x10, chip_id=0x8103,
+                display_name='iPad Air (5th gen, WiFi)'),
+    IRecvDevice(product_type='iPad13,17', hardware_model='j408ap', board_id=0x12, chip_id=0x8103,
+                display_name='iPad Air (5th gen, Celluar)'),
     IRecvDevice(product_type='iPad14,1', hardware_model='j310ap', board_id=0x04, chip_id=0x8110,
                 display_name='iPad mini (6th gen, WiFi)'),
     IRecvDevice(product_type='iPad14,2', hardware_model='j311ap', board_id=0x06, chip_id=0x8110,
@@ -365,6 +371,10 @@ IRECV_DEVICES = (
                 display_name='iMac 24-inch (M1, Two Ports, 2021)'),
     IRecvDevice(product_type='iMac21,2', hardware_model='j457ap', board_id=0x2A, chip_id=0x8103,
                 display_name='iMac 24-inch (M1, Four Ports, 2021)'),
+    IRecvDevice(product_type='Mac13,1', hardware_model='j375cap', board_id=0x04, chip_id=0x6001,
+                display_name='Mac Studio (M1 Max, 2022)'),
+    IRecvDevice(product_type='Mac13,2', hardware_model='j375dap', board_id=0x0C, chip_id=0x6002,
+                display_name='Mac Studio (M1 Ultra, 2022)'),
     # Apple T2 Coprocessor
     IRecvDevice(product_type='iBridge2,1', hardware_model='j137ap', board_id=0x0A, chip_id=0x8012,
                 display_name='Apple T2 iMacPro1,1 (j137)'),
@@ -398,4 +408,7 @@ IRECV_DEVICES = (
                 display_name='Apple T2 MacBookPro16,3 (j223)'),
     IRecvDevice(product_type='iBridge2,22', hardware_model='j215ap', board_id=0x38, chip_id=0x8012,
                 display_name='Apple T2 MacBookPro16,4 (j215)'),
+    # Apple Displays
+    IRecvDevice(product_type='AppleDisplay2,1', hardware_model='j327ap', board_id=0x22, chip_id=0x8030,
+                display_name='Studio Display'),
 )
