@@ -4,8 +4,10 @@ from datetime import timezone, timedelta
 from io import BytesIO
 
 from construct import Struct, Int32ul, Int64ul, FixedSized, GreedyRange, GreedyBytes, Enum, Switch, Padding, Padded, \
-    LazyBound, CString, Computed, Array, this, Byte, Int16ul, Pass, Bytes
+    LazyBound, CString, Computed, Array, this, Byte, Int16ul, Pass, Bytes, GreedyString
+from pykdebugparser.kd_buf_parser import RAW_VERSION2_BYTES
 
+from pymobiledevice3.exceptions import ExtractingStackshotError
 from pymobiledevice3.resources.dsc_uuid_map import get_dsc_map
 from pymobiledevice3.services.dvt.dvt_secure_socket_proxy import DvtSecureSocketProxyService
 from pymobiledevice3.services.dvt.instruments.device_info import DeviceInfo
@@ -220,7 +222,8 @@ thread_group_snapshot = Struct(
     predefined_name_substruct,
     'obj' / Struct(
         'tgs_id' / Int64ul,
-        'tgs_name' / Padded(16, CString('utf8')),
+        '_tgs_name' / FixedSized(16, GreedyString('utf8')),
+        'tgs_name' / Computed(lambda ctx: ctx._tgs_name.strip('\x00')),
         'tgs_flags' / Int64ul
     ),
 )
@@ -584,8 +587,12 @@ class CoreProfileSessionTap(Tap):
             # The stackshot is sent one per TAP creation, so we cache it.
             return self.stack_shot
         data = self._channel.receive_message()
-        while not data.startswith(STACKSHOT_HEADER):
+        while not data.startswith(STACKSHOT_HEADER) and not data.startswith(RAW_VERSION2_BYTES):
             data = self._channel.receive_message()
+
+        if data.startswith(RAW_VERSION2_BYTES):
+            raise ExtractingStackshotError()
+
         stackshot = self.parse_stackshot(data)
 
         dsc_map = get_dsc_map(str(stackshot['shared_cache_dyld_load_info']['imageUUID']))
@@ -631,7 +638,10 @@ class CoreProfileSessionTap(Tap):
 
     @staticmethod
     def get_time_config(dvt):
-        mach_absolute_time, numer, denom, _ = DeviceInfo(dvt).mach_time_info()
+        time_info = DeviceInfo(dvt).mach_time_info()
+        mach_absolute_time = time_info[0]
+        numer = time_info[1]
+        denom = time_info[2]
         usecs_since_epoch = dvt.lockdown.get_value(key='TimeIntervalSince1970') * 1000000
         return dict(
             numer=numer, denom=denom, mach_absolute_time=mach_absolute_time, usecs_since_epoch=usecs_since_epoch,
